package javax.management.remote.rmi;

import com.sun.jmx.remote.internal.ArrayNotificationBuffer;
import java.util.Set;
import java.security.Principal;
import java.rmi.server.ServerNotActiveException;
import java.rmi.server.RemoteServer;
import java.util.Iterator;
import javax.security.auth.Subject;
import com.sun.jmx.remote.security.JMXPluggableAuthenticator;
import javax.management.remote.JMXAuthenticator;
import java.rmi.Remote;
import java.io.IOException;
import java.util.Collections;
import java.util.ArrayList;
import com.sun.jmx.remote.internal.NotificationBuffer;
import java.util.Map;
import javax.management.MBeanServer;
import java.lang.ref.WeakReference;
import java.util.List;
import com.sun.jmx.remote.util.ClassLogger;
import java.io.Closeable;

public abstract class RMIServerImpl implements Closeable, RMIServer
{
    private static final ClassLogger logger;
    private final List<WeakReference<RMIConnection>> clientList;
    private ClassLoader cl;
    private MBeanServer mbeanServer;
    private final Map<String, ?> env;
    private RMIConnectorServer connServer;
    private static int connectionIdNumber;
    private NotificationBuffer notifBuffer;
    
    public RMIServerImpl(Map<String, ?> map) {
        this.clientList = new ArrayList();
        this.env = ((map == null) ? Collections.emptyMap() : map);
    }
    
    void setRMIConnectorServer(RMIConnectorServer connServer) throws IOException {
        this.connServer = connServer;
    }
    
    protected abstract void export() throws IOException;
    
    public abstract Remote toStub() throws IOException;
    
    public synchronized void setDefaultClassLoader(ClassLoader cl) {
        this.cl = cl;
    }
    
    public synchronized ClassLoader getDefaultClassLoader() {
        return this.cl;
    }
    
    public synchronized void setMBeanServer(MBeanServer mbeanServer) {
        this.mbeanServer = mbeanServer;
    }
    
    public synchronized MBeanServer getMBeanServer() {
        return this.mbeanServer;
    }
    
    @Override
    public String getVersion() {
        try {
            return "1.0 java_runtime_" + System.getProperty("java.runtime.version");
        }
        catch (SecurityException ex) {
            return "1.0 ";
        }
    }
    
    @Override
    public RMIConnection newClient(Object o) throws IOException {
        return this.doNewClient(o);
    }
    
    RMIConnection doNewClient(Object o) throws IOException {
        int traceOn = RMIServerImpl.logger.traceOn();
        if (!(traceOn == 0)) {
            RMIServerImpl.logger.trace("newClient", "making new client");
        }
        if (this.getMBeanServer() == null) {
            throw new IllegalStateException("Not attached to an MBean server");
        }
        Subject authenticate = null;
        JMXAuthenticator jmxAuthenticator = (JMXAuthenticator)this.env.get("jmx.remote.authenticator");
        if (jmxAuthenticator == null && (this.env.get("jmx.remote.x.password.file") != null || this.env.get("jmx.remote.x.login.config") != null)) {
            jmxAuthenticator = new JMXPluggableAuthenticator(this.env);
        }
        if (jmxAuthenticator != null) {
            if (!(traceOn == 0)) {
                RMIServerImpl.logger.trace("newClient", "got authenticator: " + jmxAuthenticator.getClass().getName());
            }
            try {
                authenticate = jmxAuthenticator.authenticate(o);
            }
            catch (SecurityException obj) {
                RMIServerImpl.logger.trace("newClient", "Authentication failed: " + obj);
                throw obj;
            }
        }
        if (!(traceOn == 0)) {
            if (authenticate != null) {
                RMIServerImpl.logger.trace("newClient", "subject is not null");
            }
            else {
                RMIServerImpl.logger.trace("newClient", "no subject");
            }
        }
        String connectionId = makeConnectionId(this.getProtocol(), authenticate);
        if (!(traceOn == 0)) {
            RMIServerImpl.logger.trace("newClient", "making new connection: " + connectionId);
        }
        RMIConnection client = this.makeClient(connectionId, authenticate);
        this.dropDeadReferences();
        WeakReference o2 = new WeakReference(client);
        synchronized (this.clientList) {
            this.clientList.add(o2);
        }
        this.connServer.connectionOpened(connectionId, "Connection opened", null);
        synchronized (this.clientList) {
            if (!this.clientList.contains(o2)) {
                throw new IOException("The connection is refused.");
            }
        }
        if (!(traceOn == 0)) {
            RMIServerImpl.logger.trace("newClient", "new connection done: " + connectionId);
            return client;
        }
        return client;
    }
    
    protected abstract RMIConnection makeClient(String p0, Subject p1) throws IOException;
    
    protected abstract void closeClient(RMIConnection p0) throws IOException;
    
    protected abstract String getProtocol();
    
    protected void clientClosed(RMIConnection obj) throws IOException {
        int debugOn = RMIServerImpl.logger.debugOn();
        if (!(debugOn == 0)) {
            RMIServerImpl.logger.trace("clientClosed", "client=" + obj);
        }
        if (obj == null) {
            throw new NullPointerException("Null client");
        }
        synchronized (this.clientList) {
            this.dropDeadReferences();
            Iterator<WeakReference<RMIConnection>> iterator = this.clientList.iterator();
            while (iterator.hasNext()) {
                if (((WeakReference)iterator.next()).get() == obj) {
                    iterator.remove();
                    break;
                }
                continue;
            }
        }
        if (!(debugOn == 0)) {
            RMIServerImpl.logger.trace("clientClosed", "closing client.");
        }
        this.closeClient(obj);
        if (!(debugOn == 0)) {
            RMIServerImpl.logger.trace("clientClosed", "sending notif");
        }
        this.connServer.connectionClosed(obj.getConnectionId(), "Client connection closed", null);
        if (!(debugOn == 0)) {
            RMIServerImpl.logger.trace("clientClosed", "done");
        }
        return;
    }
    
    @Override
    public synchronized void close() throws IOException {
        int traceOn = RMIServerImpl.logger.traceOn();
        int debugOn = RMIServerImpl.logger.debugOn();
        if (!(traceOn == 0)) {
            RMIServerImpl.logger.trace("close", "closing");
        }
        IOException ex = null;
        try {
            if (!(debugOn == 0)) {
                RMIServerImpl.logger.debug("close", "closing Server");
            }
            this.closeServer();
        }
        catch (IOException ex2) {
            if (!(traceOn == 0)) {
                RMIServerImpl.logger.trace("close", "Failed to close server: " + ex2);
            }
            if (!(debugOn == 0)) {
                RMIServerImpl.logger.debug("close", ex2);
            }
            ex = ex2;
        }
        if (!(debugOn == 0)) {
            RMIServerImpl.logger.debug("close", "closing Clients");
        }
        while (true) {
            synchronized (this.clientList) {
                if (!(debugOn == 0)) {
                    RMIServerImpl.logger.debug("close", "droping dead references");
                }
                this.dropDeadReferences();
                if (!(debugOn == 0)) {
                    RMIServerImpl.logger.debug("close", "client count: " + this.clientList.size());
                }
                if (this.clientList.size() == 0) {
                    break;
                }
                Iterator<WeakReference<RMIConnection>> iterator = this.clientList.iterator();
                while (iterator.hasNext()) {
                    RMIConnection rmiConnection = (RMIConnection)((WeakReference)iterator.next()).get();
                    iterator.remove();
                    if (rmiConnection != null) {
                        try {
                            rmiConnection.close();
                        }
                        catch (IOException ex3) {
                            if (!(traceOn == 0)) {
                                RMIServerImpl.logger.trace("close", "Failed to close client: " + ex3);
                            }
                            if (!(debugOn == 0)) {
                                RMIServerImpl.logger.debug("close", ex3);
                            }
                            if (ex == null) {
                                ex = ex3;
                            }
                        }
                        break;
                    }
                    continue;
                }
            }
        }
        if (this.notifBuffer != null) {
            this.notifBuffer.dispose();
        }
        if (ex != null) {
            if (!(traceOn == 0)) {
                RMIServerImpl.logger.trace("close", "close failed.");
            }
            throw ex;
        }
        else {
            if (!(traceOn == 0)) {
                RMIServerImpl.logger.trace("close", "closed.");
            }
            return;
        }
    }
    
    protected abstract void closeServer() throws IOException;
    
    private static synchronized String makeConnectionId(String str, Subject subject) {
        ++RMIServerImpl.connectionIdNumber;
        String s = "";
        try {
            s = RemoteServer.getClientHost();
            if (s.contains(":")) {
                s = "[" + s + "]";
            }
        }
        catch (ServerNotActiveException t) {
            RMIServerImpl.logger.trace("makeConnectionId", "getClientHost", t);
        }
        StringBuilder obj = new StringBuilder();
        obj.append(str).append(":");
        if (s.length() > 0) {
            obj.append("//").append(s);
        }
        obj.append(" ");
        if (subject != null) {
            Set<Principal> principals = subject.getPrincipals();
            String str2 = "";
            for (Iterator iterator = principals.iterator(); iterator.hasNext(); str2 = ";") {
                obj.append(str2).append(((Principal)iterator.next()).getName().replace(' ', '_').replace(';', ':'));
            }
        }
        obj.append(" ").append(RMIServerImpl.connectionIdNumber);
        if (RMIServerImpl.logger.traceOn()) {
            RMIServerImpl.logger.trace("newConnectionId", "connectionId=" + obj);
        }
        return obj.toString();
    }
    
    private void dropDeadReferences() {
        synchronized (this.clientList) {
            Iterator<WeakReference<RMIConnection>> iterator = this.clientList.iterator();
            while (iterator.hasNext()) {
                if (((WeakReference)iterator.next()).get() == null) {
                    iterator.remove();
                }
                else {
                    continue;
                }
            }
        }
    }
    
    synchronized NotificationBuffer getNotifBuffer() {
        if (this.notifBuffer == null) {
            this.notifBuffer = ArrayNotificationBuffer.getNotificationBuffer(this.mbeanServer, this.env);
        }
        return this.notifBuffer;
    }
    
    static {
        logger = new ClassLogger("javax.management.remote.rmi", "RMIServerImpl");
    }
}
